@@ -22,19 +22,15 @@ from meta import name as appname, version as appversion
 from util_os import launch_file, waccess
 from wxaddons import CustomEvent
 from wxMeasureFrame import MeasureFrame
-from wxwindows import BaseApp, BaseFrame, FlatShadedButton, numpad_keycodes
+from wxwindows import (BaseApp, BaseFrame, FlatShadedButton,
+					   numpad_keycodes, nav_keycodes, processing_keycodes,
+					   wx_Panel)
 import colormath
 import config
 import localization as lang
 import report
 
 BGCOLOUR = wx.Colour(0x33, 0x33, 0x33)
-BLACK = wx.Colour(0, 0, 0)
-BORDERCOLOUR = wx.Colour(0x22, 0x22, 0x22)
-FGCOLOUR = wx.Colour(0x99, 0x99, 0x99)
-LIGHTGRAY = wx.Colour(0xcc, 0xcc, 0xcc)
-MEDIUMGRAY = wx.Colour(0x80, 0x80, 0x80)
-WHITE = wx.Colour(0xff, 0xff, 0xff)
 
 
 class FlatShadedNumberedButton(FlatShadedButton):
@@ -47,6 +43,10 @@ class FlatShadedNumberedButton(FlatShadedButton):
 								  style, validator, name, bgcolour, fgcolour)
 		self.index = index
 
+	def OnGainFocus(self, event):
+		self.TopLevelParent.index = self.index
+		FlatShadedButton.OnGainFocus(self, event)
+
 
 class DisplayUniformityFrame(BaseFrame):
 
@@ -67,13 +67,13 @@ class DisplayUniformityFrame(BaseFrame):
 		
 		self.rows = rows
 		self.cols = cols
-		self.colors = (WHITE, wx.Colour(192, 192, 192),
-							  wx.Colour(128, 128, 128), wx.Colour(64, 64, 64))
+		self.colors = (wx.WHITE, wx.Colour(192, 192, 192),
+					   wx.Colour(128, 128, 128), wx.Colour(64, 64, 64))
 		self.labels = {}
 		self.panels = []
 		self.buttons = []
 		for index in xrange(rows * cols):
-			panel = wx.Panel(self, style=wx.BORDER_SIMPLE)
+			panel = wx_Panel(self, style=wx.BORDER_SIMPLE)
 			panel.SetBackgroundColour(BGCOLOUR)
 			sizer = wx.BoxSizer(wx.VERTICAL)
 			panel.SetSizer(sizer)
@@ -81,12 +81,11 @@ class DisplayUniformityFrame(BaseFrame):
 			button = FlatShadedNumberedButton(panel,
 											  label=lang.getstr("measure"),
 											  bitmap=getbitmap("theme/icons/10x10/record"),
-											  bgcolour=LIGHTGRAY,
-											  fgcolour=BLACK, index=index)
+											  index=index)
 			button.Bind(wx.EVT_BUTTON, self.measure)
 			self.buttons.append(button)
 			label = wx.StaticText(panel)
-			label.SetForegroundColour(WHITE)
+			label.SetForegroundColour(wx.WHITE)
 			self.labels[index] = label
 			sizer.Add(label, 1, wx.ALIGN_CENTER)
 			sizer.Add(button, 0, wx.ALIGN_BOTTOM | wx.ALIGN_CENTER | wx.BOTTOM |
@@ -96,18 +95,24 @@ class DisplayUniformityFrame(BaseFrame):
 		self.disable_buttons()
 		
 		self.keyhandler = keyhandler
-		if sys.platform in ("darwin", "win32"):
-			# Use an accelerator table for space, 0-9, A-Z, numpad
-			keycodes = [ord(" ")] + range(ord("0"),
-										  ord("9")) + range(ord("A"),
-															ord("Z")) + numpad_keycodes
-			self.id_to_keycode = {}
+		self.id_to_keycode = {}
+		if sys.platform == "darwin":
+			# Use an accelerator table for tab, space, 0-9, A-Z, numpad,
+			# navigation keys and processing keys
+			keycodes = [wx.WXK_TAB, wx.WXK_SPACE]
+			keycodes.extend(range(ord("0"), ord("9")))
+			keycodes.extend(range(ord("A"), ord("Z")))
+			keycodes.extend(numpad_keycodes)
+			keycodes.extend(nav_keycodes)
+			keycodes.extend(processing_keycodes)
 			for keycode in keycodes:
-				self.id_to_keycode[wx.NewId()] = keycode
+				self.id_to_keycode[wx.Window.NewControlId()] = keycode
 			accels = []
 			for id, keycode in self.id_to_keycode.iteritems():
 				self.Bind(wx.EVT_MENU, self.key_handler, id=id)
 				accels.append((wx.ACCEL_NORMAL, keycode, id))
+				if keycode == wx.WXK_TAB:
+					accels.append((wx.ACCEL_SHIFT, keycode, id))
 			self.SetAcceleratorTable(wx.AcceleratorTable(accels))
 		else:
 			self.Bind(wx.EVT_CHAR_HOOK, self.key_handler)
@@ -144,6 +149,13 @@ class DisplayUniformityFrame(BaseFrame):
 	def OnDestroy(self, event):
 		self.stop_timer()
 		del self.timer
+		if hasattr(wx.Window, "UnreserveControlId"):
+			for id in self.id_to_keycode.iterkeys():
+				if id < 0:
+					try:
+						wx.Window.UnreserveControlId(id)
+					except wx.wxAssertionError, exception:
+						safe_print(exception)
 		
 	def OnMove(self, event):
 		pass
@@ -168,8 +180,9 @@ class DisplayUniformityFrame(BaseFrame):
 			self.disable_buttons()
 			wx.CallAfter(self.Maximize)
 		wx.Frame.Show(self, show)
+		self.panels[0].SetFocus()
 	
-	def Update(self, value, msg=""):
+	def UpdateProgress(self, value, msg=""):
 		return self.Pulse(msg)
 	
 	def UpdatePulse(self, msg=""):
@@ -213,15 +226,23 @@ class DisplayUniformityFrame(BaseFrame):
 			keycode = event.GetKeyCode()
 		elif event.GetEventType() == wx.EVT_MENU.typeId:
 			keycode = self.id_to_keycode.get(event.GetId())
-		if keycode is not None:
-			if self.has_worker_subprocess():
-				if keycode == 27 or chr(keycode) == "Q":
+		if keycode == wx.WXK_TAB:
+			self.global_navigate() or event.Skip()
+		elif keycode >= 0:
+			if self.has_worker_subprocess() and keycode < 256:
+				if keycode == wx.WXK_ESCAPE or chr(keycode) == "Q":
 					# ESC or Q
 					self.worker.abort_subprocess()
-				elif self.index > -1 and not self.is_measuring:
+				elif (self.index > -1 and not self.is_measuring and
+					  (not isinstance(self.FindFocus(), wx.Control) or
+					   keycode != wx.WXK_SPACE)):
 					# Any other key
 					self.measure(CustomEvent(wx.EVT_BUTTON.typeId,
 											 self.buttons[self.index]))
+				else:
+					event.Skip()
+			else:
+				event.Skip()
 		else:
 			event.Skip()
 	
@@ -275,6 +296,7 @@ class DisplayUniformityFrame(BaseFrame):
 				self.show_cursor()
 				self.enable_buttons()
 				self.buttons[self.index].Show()
+				self.buttons[self.index].SetFocus()
 				self.buttons[self.index].SetBitmap(getbitmap("theme/icons/16x16/checkmark"))
 				self.panels[self.index].SetBackgroundColour(BGCOLOUR)
 				self.panels[self.index].Refresh()
@@ -350,7 +372,7 @@ class DisplayUniformityFrame(BaseFrame):
 	
 	def _setup(self):
 		self.logger.info("-" * 80)
-		self.index = -1
+		self.index = 0
 		self.is_measuring = False
 		self.keepGoing = True
 		self.last_error = None
@@ -409,7 +431,7 @@ if __name__ == "__main__":
 	lang.init()
 	lang.update_defaults()
 	app = BaseApp(0)
-	app.TopWindow = DisplayUniformityFrame(start_timer=False, cols=3)
+	app.TopWindow = DisplayUniformityFrame(start_timer=False, rows=3, cols=3)
 	app.TopWindow.worker = Worker()
 	app.TopWindow.Show()
 	i = 0
